from DesOptPy.DesOptPy import *
from DesOptPy.tools import *
from DesOptPy.plotting import *

# from DesOptPy.FunctionAggregation import *


__title__ = 'DESign OPTimization in PYthon'
__shorttitle__ = 'DesOptPy'
__version__ = '2.0 development'
# __all__ = ["DesOpt"]
__author__ = 'E. J. Wehrle'
__copyright__ = 'Copyright 2021 E. J. Wehrle'
__email__ = 'Erich.Wehrle(a)unibz.it'
__url__ = 'github.com/edub/DesOptPy2'
# TODO decide on license for new version
__license__ = 'TBD'  # "GNU Lesser General Public License"


def PrintDesOptPy():
    print()
    print(__title__ + ' - ' + __shorttitle__)
    print('Version:    ' + __version__)
    print('License:    ' + __license__)
    print('Internet:   ' + __url__)
    print('\n')


PrintDesOptPy()
